@@ -17,13 +17,22 @@
 
 //! Decimal related utils
 
+use crate::datatypes::{
+    DECIMAL128_MAX_PRECISION, DECIMAL128_MAX_SCALE, DECIMAL256_MAX_PRECISION,
+    DECIMAL256_MAX_SCALE,
+};
 use crate::error::{ArrowError, Result};
 use num::bigint::BigInt;
+use num::Signed;
 use std::cmp::{min, Ordering};
 
 pub trait BasicDecimal: PartialOrd + Ord + PartialEq + Eq {
     /// The bit-width of the internal representation.
     const BIT_WIDTH: usize;
+    /// The maximum precision.
+    const MAX_PRECISION: usize;
+    /// The maximum scale.
+    const MAX_SCALE: usize;
 
     /// Tries to create a decimal value from precision, scale and bytes.
     /// If the length of bytes isn't same as the bit width of this decimal,
@@ -36,6 +45,21 @@ pub trait BasicDecimal: PartialOrd + Ord + PartialEq + Eq {
     where
         Self: Sized,
     {
+        if precision > Self::MAX_PRECISION {
+            return Err(ArrowError::InvalidArgumentError(format!(
+                "precision {} is greater than max {}",
+                precision,
+                Self::MAX_PRECISION
+            )));
+        }
+        if scale > Self::MAX_SCALE {
+            return Err(ArrowError::InvalidArgumentError(format!(
+                "scale {} is greater than max {}",
+                scale,
+                Self::MAX_SCALE
+            )));
+        }
+
         if precision < scale {
             return Err(ArrowError::InvalidArgumentError(format!(
                 "Precision {} is less than scale {}",
@@ -138,10 +162,30 @@ pub struct Decimal256 {
     value: [u8; 32],
 }
 
+impl Decimal256 {
+    /// Constructs a `Decimal256` value from a `BigInt`.
+    pub fn from_big_int(
+        num: &BigInt,
+        precision: usize,
+        scale: usize,
+    ) -> Result<Decimal256> {
+        let mut bytes = if num.is_negative() {
+            vec![255; 32]
+        } else {
+            vec![0; 32]
+        };
+        let num_bytes = &num.to_signed_bytes_le();
+        bytes[0..num_bytes.len()].clone_from_slice(num_bytes);
+        Decimal256::try_new_from_bytes(precision, scale, &bytes)
+    }
+}
+
 macro_rules! def_decimal {
-    ($ty:ident, $bit:expr) => {
+    ($ty:ident, $bit:expr, $max_p:expr, $max_s:expr) => {
         impl BasicDecimal for $ty {
             const BIT_WIDTH: usize = $bit;
+            const MAX_PRECISION: usize = $max_p;
+            const MAX_SCALE: usize = $max_s;
 
             fn new(precision: usize, scale: usize, bytes: &[u8]) -> Self {
                 $ty {
@@ -171,7 +215,7 @@ macro_rules! def_decimal {
                     "Cannot compare two Decimals with different scale: {}, {}",
                     self.scale, other.scale
                 );
-                self.value.partial_cmp(&other.value)
+                Some(singed_cmp_le_bytes(&self.value, &other.value))
             }
         }
 
@@ -182,7 +226,7 @@ macro_rules! def_decimal {
                     "Cannot compare two Decimals with different scale: {}, {}",
                     self.scale, other.scale
                 );
-                self.value.cmp(&other.value)
+                singed_cmp_le_bytes(&self.value, &other.value)
             }
         }
 
@@ -201,12 +245,67 @@ macro_rules! def_decimal {
     };
 }
 
-def_decimal!(Decimal128, 128);
-def_decimal!(Decimal256, 256);
+// compare two signed integer which are encoded with little endian.
+// left bytes and right bytes must have the same length.
+fn singed_cmp_le_bytes(left: &[u8], right: &[u8]) -> Ordering {
+    assert_eq!(
+        left.len(),
+        right.len(),
+        "Can't compare bytes array with different len: {}, {}",
+        left.len(),
+        right.len()
+    );
+    assert_ne!(left.len(), 0, "Can't compare bytes array of length 0");
+    let len = left.len();
+    // the sign bit is 1, the value is negative
+    let left_negative = left[len - 1] >= 0x80_u8;
+    let right_negative = right[len - 1] >= 0x80_u8;
+    if left_negative != right_negative {
+        return match left_negative {
+            true => {
+                // left is negative value
+                // right is positive value
+                Ordering::Less
+            }
+            false => Ordering::Greater,
+        };
+    }
+    for i in 0..len {
+        let l_byte = left[len - 1 - i];
+        let r_byte = right[len - 1 - i];
+        match l_byte.cmp(&r_byte) {
+            Ordering::Less => {
+                return Ordering::Less;
+            }
+            Ordering::Greater => {
+                return Ordering::Greater;
+            }
+            Ordering::Equal => {}
+        }
+    }
+    Ordering::Equal
+}
+
+def_decimal!(
+    Decimal128,
+    128,
+    DECIMAL128_MAX_PRECISION,
+    DECIMAL128_MAX_SCALE
+);
+def_decimal!(
+    Decimal256,
+    256,
+    DECIMAL256_MAX_PRECISION,
+    DECIMAL256_MAX_SCALE
+);
 
 #[cfg(test)]
 mod tests {
-    use crate::util::decimal::{BasicDecimal, Decimal128, Decimal256};
+    use crate::util::decimal::{
+        singed_cmp_le_bytes, BasicDecimal, Decimal128, Decimal256,
+    };
+    use num::{BigInt, Num};
+    use rand::random;
 
     #[test]
     fn decimal_128_to_string() {
@@ -281,10 +380,10 @@ mod tests {
         // smaller than i128 minimum
         bytes = vec![255; 32];
         bytes[31] = 128;
-        let value = Decimal256::try_new_from_bytes(79, 4, &bytes).unwrap();
+        let value = Decimal256::try_new_from_bytes(76, 4, &bytes).unwrap();
         assert_eq!(
             value.to_string(),
-            "-5744373177007483132341216834415376678658315645522012356644966081642565415.7313"
+            "-574437317700748313234121683441537667865831564552201235664496608164256541.5731"
         );
 
         bytes = vec![255; 32];
@@ -302,4 +401,79 @@ mod tests {
         let integer = i128_func(value);
         assert_eq!(integer, 100);
     }
+
+    #[test]
+    fn bigint_to_decimal256() {
+        let num = BigInt::from_str_radix("123456789", 10).unwrap();
+        let value = Decimal256::from_big_int(&num, 30, 2).unwrap();
+        assert_eq!(value.to_string(), "1234567.89");
+
+        let num = BigInt::from_str_radix("-5744373177007483132341216834415376678658315645522012356644966081642565415731", 10).unwrap();
+        let value = Decimal256::from_big_int(&num, 76, 4).unwrap();
+        assert_eq!(value.to_string(), "-574437317700748313234121683441537667865831564552201235664496608164256541.5731");
+    }
+
+    #[test]
+    fn test_lt_cmp_byte() {
+        for _i in 0..100 {
+            let left = random::<i128>();
+            let right = random::<i128>();
+            let result = singed_cmp_le_bytes(
+                left.to_le_bytes().as_slice(),
+                right.to_le_bytes().as_slice(),
+            );
+            assert_eq!(left.cmp(&right), result);
+        }
+        for _i in 0..100 {
+            let left = random::<i32>();
+            let right = random::<i32>();
+            let result = singed_cmp_le_bytes(
+                left.to_le_bytes().as_slice(),
+                right.to_le_bytes().as_slice(),
+            );
+            assert_eq!(left.cmp(&right), result);
+        }
+    }
+
+    #[test]
+    fn compare_decimal128() {
+        let v1 = -100_i128;
+        let v2 = 10000_i128;
+        let right = Decimal128::new_from_i128(20, 3, v2);
+        for v in v1..v2 {
+            let left = Decimal128::new_from_i128(20, 3, v);
+            assert!(left < right);
+        }
+
+        for _i in 0..100 {
+            let left = random::<i128>();
+            let right = random::<i128>();
+            let left_decimal = Decimal128::new_from_i128(38, 2, left);
+            let right_decimal = Decimal128::new_from_i128(38, 2, right);
+            assert_eq!(left < right, left_decimal < right_decimal);
+            assert_eq!(left == right, left_decimal == right_decimal)
+        }
+    }
+
+    #[test]
+    fn compare_decimal256() {
+        let v1 = -100_i128;
+        let v2 = 10000_i128;
+        let right = Decimal256::from_big_int(&BigInt::from(v2), 75, 2).unwrap();
+        for v in v1..v2 {
+            let left = Decimal256::from_big_int(&BigInt::from(v), 75, 2).unwrap();
+            assert!(left < right);
+        }
+
+        for _i in 0..100 {
+            let left = random::<i128>();
+            let right = random::<i128>();
+            let left_decimal =
+                Decimal256::from_big_int(&BigInt::from(left), 75, 2).unwrap();
+            let right_decimal =
+                Decimal256::from_big_int(&BigInt::from(right), 75, 2).unwrap();
+            assert_eq!(left < right, left_decimal < right_decimal);
+            assert_eq!(left == right, left_decimal == right_decimal)
+        }
+    }
 }
