@@ -119,7 +119,7 @@
 //!}
 //! ```
 
-experimental_mod!(array_reader);
+experimental!(mod array_reader);
 pub mod arrow_reader;
 pub mod arrow_writer;
 mod buffer;
@@ -128,7 +128,7 @@ mod buffer;
 pub mod async_reader;
 
 mod record_reader;
-experimental_mod!(schema);
+experimental!(mod schema);
 
 pub use self::arrow_reader::ArrowReader;
 pub use self::arrow_reader::ParquetFileArrowReader;
